
#include "RenderTexture.h"

#define CHECK_FRAMEBUFFER_STATUS() \
{\
 GLenum status; \
 status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); \
 wxLogMessage(_T("%x\n"), status);\
 switch(status) { \
 case GL_FRAMEBUFFER_COMPLETE_EXT: \
   wxLogMessage(_T("framebuffer complete!\n"));\
   break; \
 case GL_FRAMEBUFFER_UNSUPPORTED_EXT: \
   wxLogMessage(_T("framebuffer GL_FRAMEBUFFER_UNSUPPORTED_EXT\n"));\
    /* you gotta choose different formats */ \
   assert(0); \
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT: \
   wxLogMessage(_T("framebuffer INCOMPLETE_ATTACHMENT\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT: \
   wxLogMessage(_T("framebuffer FRAMEBUFFER_MISSING_ATTACHMENT\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT: \
   wxLogMessage(_T("framebuffer FRAMEBUFFER_DIMENSIONS\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT: \
   wxLogMessage(_T("framebuffer INCOMPLETE_DUPLICATE_ATTACHMENT\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT: \
   wxLogMessage(_T("framebuffer INCOMPLETE_FORMATS\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT: \
   wxLogMessage(_T("framebuffer INCOMPLETE_DRAW_BUFFER\n"));\
   break; \
 case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT: \
   wxLogMessage(_T("framebuffer INCOMPLETE_READ_BUFFER\n"));\
   break; \
 case GL_FRAMEBUFFER_BINDING_EXT: \
   wxLogMessage(_T("framebuffer BINDING_EXT\n"));\
   break; \
/* 
 case GL_FRAMEBUFFER_STATUS_ERROR_EXT: \
   wxLogMessage(_T("framebuffer STATUS_ERROR\n");\
   break; \
*/ \
 default: \
   /* programming error; will fail on all hardware */ \
   assert(0); \
 }\
}


void RenderTexture::InitGL()
{
	if (supportDrawRangeElements || supportVBO) {
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	}

	if(supportAntiAlias && useAntiAlias)
		glEnable(GL_MULTISAMPLE_ARB);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	//glColorMaterial(GL_FRONT, GL_SPECULAR);
	
	/*
	//glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE);
	glTexEnvf(GL_TEXTURE_ENV,GL_RGB_SCALE, 1.000000);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_RGB, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE2_RGB, GL_CONSTANT);
	glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB, GL_MODULATE);
	glTexEnvf(GL_TEXTURE_ENV,GL_ALPHA_SCALE, 1.000000);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_ALPHA, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_ALPHA, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE2_ALPHA, GL_CONSTANT);
	glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_ALPHA, GL_REPLACE);	
	*/

	glAlphaFunc(GL_GEQUAL, 0.4f);
	glDepthFunc(GL_LEQUAL);
	glMaterialf(GL_FRONT, GL_SHININESS, 18.0f);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glPixelStorei(GL_PACK_ALIGNMENT, 4);
	glPixelStorei(GL_PACK_SWAP_BYTES, false);
	glPixelStorei(GL_PACK_LSB_FIRST, false);

	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
}

void RenderTexture::Init(HWND g_hWnd, int width, int height, bool fbomode)
{
	canvas_hWnd = g_hWnd;
	canvas_hDC = wglGetCurrentDC();
	canvas_hRC = wglGetCurrentContext();

	texID = 0;
	FBO = fbomode;

	int screenSize[4];
	glGetIntegerv(GL_VIEWPORT, screenSize);

	if (FBO)
		texFormat = 0x84F5; // GL_TEXTURE_RECTANGLE_ARB
	else
		texFormat = 0x0DE1; // GL_TEXTURE_2D

	if (width == 0 || height == 0) {
		nWidth = screenSize[2];
		nHeight = screenSize[3];
	} else {
		nWidth = width;
		nHeight = height;
	}

	// If non-power-of-two textures are supported, force them to be power-of-two of the max texture size.
	if (!supportNPOT) {
		GLint texSize; 
		glGetIntegerv(GL_MAX_TEXTURE_SIZE, &texSize); 

		bool end=false;
		int curMax;
		for (int i=5; !end; i++) {
			if (powf(2,(float)i) < texSize) {
				curMax = powf(2,(float)i);
			} else {
				end = true;
			}
		}

		nWidth = nHeight = curMax;
	}

	if (FBO) { // Frame Buffer OBject mode (newer, better, only supported by newer cards and drivers though)
		wxLogMessage(_T("GFX Info: Initialising FrameBufferObject."));

		// Create a frame-buffer object and a render-buffer object...
		glGenFramebuffersEXT(1, &frameBuffer );
		glGenRenderbuffersEXT(1, &depthRenderBuffer );

		// Initialize the render-buffer for usage as a depth buffer.
		// We don't really need this to render things into the frame-buffer object,
		// but without it the geometry will not be sorted properly.
		glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, depthRenderBuffer );
		glRenderbufferStorageEXT( GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, nWidth, nHeight);

		GLenum status = glCheckFramebufferStatusEXT( GL_FRAMEBUFFER_EXT );

		// Check for errors...
		if (glCheckFramebufferStatusEXT( GL_FRAMEBUFFER_EXT ) == GL_FRAMEBUFFER_COMPLETE_EXT)
			wxLogMessage(_T("GFX Info: FrameBuffer created!"));
		else
			wxLogMessage(_T("GFX Info: FrameBufferObject is not supported by this video card!"));

		// This is our dynamic texture, which will be loaded with new pixel data
		// after we're finshed rendering to the p-buffer.
		glGenTextures(1, &texID);
		glBindTexture(texFormat, texID);
		glTexImage2D(texFormat, 0, GL_RGBA, nWidth, nHeight, 0, GL_RGBA, GL_FLOAT, 0 );
		glTexParameteri(texFormat, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(texFormat, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

	} else { // Pixel Buffer Mode
		wxLogMessage(_T("GFX Info: Attempting to create PixelBuffer."));

		//-------------------------------------------------------------------------
		// Create a p-buffer for off-screen rendering.
		//-------------------------------------------------------------------------
		
		// Define the minimum pixel format requirements we will need for our 
		// p-buffer. A p-buffer is just like a frame buffer, it can have a depth 
		// buffer associated with it and it can be double buffered.
		int pf_attr[] =
		{
			WGL_SUPPORT_OPENGL_ARB, TRUE,       // P-buffer will be used with OpenGL
			WGL_DRAW_TO_PBUFFER_ARB, TRUE,      // Enable render to p-buffer
			WGL_BIND_TO_TEXTURE_RGBA_ARB, TRUE, // P-buffer will be used as a texture
			WGL_RED_BITS_ARB, 8,                // At least 8 bits for RED channel
			WGL_GREEN_BITS_ARB, 8,              // At least 8 bits for GREEN channel
			WGL_BLUE_BITS_ARB, 8,               // At least 8 bits for BLUE channel
			WGL_ALPHA_BITS_ARB, 8,              // At least 8 bits for ALPHA channel
			WGL_DEPTH_BITS_ARB, 16,             // At least 16 bits for depth buffer
			WGL_DOUBLE_BUFFER_ARB, FALSE,       // We don't require double buffering
			0                                   // Zero terminates the list
		};

		unsigned int count = 0;
		int pixelFormat;
		// g_hdc = wxglcanvas hdc
		wglChoosePixelFormatARB(canvas_hDC, (const int*)pf_attr, NULL, 1, &pixelFormat, &count);

		if (count == 0){
			wxLogMessage(_T("GFX Error: PBuffer could not find an acceptable pixel format!"));
			return;
		}

		// Set some p-buffer attributes so that we can use this p-buffer as a
		// 2D RGBA texture target.
		int pb_attr[] =
		{
			WGL_TEXTURE_FORMAT_ARB, WGL_TEXTURE_RGBA_ARB, // Our p-buffer will have a texture format of RGBA
			WGL_TEXTURE_TARGET_ARB, WGL_TEXTURE_2D_ARB,   // Of texture target will be GL_TEXTURE_2D
			0                                             // Zero terminates the list
		};

		// Create the p-buffer...
		hPBuffer = wglCreatePbufferARB( canvas_hDC, pixelFormat, nWidth, nHeight, pb_attr );
		hDC      = wglGetPbufferDCARB( hPBuffer );
		hRC      = wglCreateContext( hDC );

		if (!hPBuffer) {
			wxLogMessage(_T("GFX Error: Could not create the PBuffer."));
			return;
		} else
			wxLogMessage(_T("GFX Info: Successfully created the PBuffer."));

		int h, w;
		wglQueryPbufferARB( hPBuffer, WGL_PBUFFER_HEIGHT_ARB, &h );
		wglQueryPbufferARB( hPBuffer, WGL_PBUFFER_WIDTH_ARB, &w );

		if (h!=nHeight || w!=nWidth) {
			wxLogMessage(_T("GFX Error: The width and height of the created PBuffer don't match the requirements!"));
			return;
		}

		if (!wglShareLists(canvas_hRC, hRC)) {
			wxLogMessage(_T("Error: Call to wglShareLists() failed for PBuffer!"));
		}

		// We were successful in creating a p-buffer. We can now make its context 
		// current and set it up just like we would a regular context 
		// attached to a window.
		if (!wglMakeCurrent(hDC, hRC)) {
			wxLogMessage(_T("GFX Error: Could not make the PBuffer's context current!"));
		}

		// Setup OpenGL RenderState
		InitGL();

		// This is our dynamic texture, which will be loaded with new pixel data
		// after we're finshed rendering to the p-buffer.
		glGenTextures(1, &texID);
		glBindTexture(texFormat, texID);
		glTexImage2D(texFormat, 0, GL_RGBA, nWidth, nHeight, 0, GL_RGBA, GL_FLOAT, 0 );
		glTexParameteri(texFormat, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(texFormat, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

		// Now set the current context back to original.
		if (!wglMakeCurrent(canvas_hDC, canvas_hRC)) {
			wxLogMessage(_T("GFX Error: Could not return the context back to the wxGLCanvas!"));
		}
	}
}

void RenderTexture::BindTexture()
{
	glBindTexture(texFormat, texID);

	if (!FBO) {
		if(!wglBindTexImageARB(hPBuffer, WGL_FRONT_LEFT_ARB )) {
			wxLogMessage(_T("GFX Error: Could not bind PixelBuffer to render texture!"));
		}
	}
}


void RenderTexture::UnBindTexture()
{
	if (!FBO) {
		if(!wglReleaseTexImageARB(hPBuffer, WGL_FRONT_LEFT_ARB ) ) {
			wxLogMessage(_T("GFX Error: Could not release PixelBuffer from render texture!"));
		}
	}
}

void RenderTexture::BeginRender()
{
	if (FBO) {

		// Bind the frame-buffer object and attach to it a render-buffer object 
		// set up as a depth-buffer.
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, frameBuffer );
		//glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, depthRenderBuffer );
		glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, texFormat, texID, 0 );
		glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, depthRenderBuffer );

	} else {
		int flag = 0;
		wglQueryPbufferARB(hPBuffer, WGL_PBUFFER_LOST_ARB, &flag );

		if( flag != 0 ) {
			wxLogMessage(_T("GFX Error: The PixelBuffer was lost!"));
			return;
		}

		if (!wglMakeCurrent(hDC, hRC))
			wxLogMessage(_T("GFX Error: Could not make the PBuffer's context current!"));
	}
}

void RenderTexture::EndRender()
{
	if (FBO) {
		// Unbind the frame-buffer and render-buffer objects.
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
		glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, 0 );

	} else { //PBuffer
		if (!wglMakeCurrent(canvas_hDC, canvas_hRC)) {
			wxLogMessage(_T("GFX Error: Could not return the context back to the wxGLCanvas!"));
		}
	}
}

void RenderTexture::Shutdown()
{
	glDeleteTextures( 1, &texID );
	texID = 0;

	if (FBO) {
		glDeleteFramebuffersEXT(1, &frameBuffer);
		glDeleteRenderbuffersEXT(1, &depthRenderBuffer);
	} else {
		// Don't forget to clean up after our p-buffer...
		if(hRC != NULL ) {
			wglMakeCurrent(hDC, hRC);
			wglDeleteContext(hRC);
			wglReleasePbufferDCARB(hPBuffer, hDC);
			wglDestroyPbufferARB(hPBuffer);
			hRC = NULL;
		}

		if (hDC != NULL ) {
			ReleaseDC(canvas_hWnd, hDC);
			hDC = NULL;
		}

		wglMakeCurrent(canvas_hDC, canvas_hRC);
	}
}

