
#include "modelexport.h"
#include "lwoheader.h"
#include "ms3dheader.h"
#include "3dsheader.h"
#include "modelcanvas.h"

#include "CxImage/ximage.h"

unsigned short numVerts = 0;
unsigned short numFaces = 0;
unsigned short numGroups = 0;

unsigned short vertIndex = 0;
unsigned short grpIndex = 0;

struct Vertex3f {
	float x;
	float y;
	float z;
};

struct ModelData {
	Vertex3f vertex;
	float tu;
	float tv;
	Vertex3f normal;
	unsigned short groupIndex;
};

struct GroupData {
	ModelRenderPass p;
	Model *m;
};

ModelData *verts = NULL;
GroupData *groups = NULL;

// 2 methods to go,  just export the entire m2 model.
// or use our "drawing" routine to export only whats being drawn.

template <typename T>
T reverse_endian(T n) // for 32bits
{
	_asm
	{
		mov EAX, n;
		bswap EAX;
		mov n, EAX;
	}

	return n;
}

unsigned short ByteSwap16 (unsigned short nValue) // 16bit
{
   return (((nValue>> 8)) | (nValue << 8));
}


void SaveTexture(wxString fn)
{
	unsigned char *pixels = NULL;

	GLint width, height;
	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &width);
	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &height);

	pixels = new unsigned char[width * height * 4];

	glGetTexImage(GL_TEXTURE_2D, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, pixels);

	CxImage *newImage = new CxImage(0);
	newImage->CreateFromArray(pixels, width, height, 32, (width*4), true);


	if (fn.Last() == 'g')
		newImage->Save(fn, CXIMAGE_FORMAT_PNG);
	else
		newImage->Save(fn, CXIMAGE_FORMAT_TGA);

	newImage->Destroy();
	wxDELETE(newImage);
	wxDELETEA(pixels);
}

void AddCount(Model *m)
{
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			numGroups++;
			
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				numVerts++;
			}
		}
	}
}

void AddVertices(Model *m)
{
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {			
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];
				
				verts[vertIndex].vertex.x = m->vertices[a].x;
				verts[vertIndex].vertex.y = m->vertices[a].y;
				verts[vertIndex].vertex.z = m->vertices[a].z;

				verts[vertIndex].tu = m->origVertices[a].texcoords.x;
				verts[vertIndex].tv = m->origVertices[a].texcoords.y;
				
				if (supportVBO) {
					verts[vertIndex].normal.x = m->vertices[m->header.nVertices + a].x;
					verts[vertIndex].normal.y = m->vertices[m->header.nVertices + a].y;
					verts[vertIndex].normal.z = m->vertices[m->header.nVertices + a].z;
				} else {
					verts[vertIndex].normal.x = m->normals[a].x;
					verts[vertIndex].normal.y = m->normals[a].y;
					verts[vertIndex].normal.z = m->normals[a].z;
				}

				verts[vertIndex].groupIndex = grpIndex;

				vertIndex++;
			}
			groups[grpIndex].p = p;
			groups[grpIndex].m = m;
			grpIndex++;
		}
	}
}

void InitCommon(Attachment *att)
{
	if (verts)
		wxDELETEA(verts);
	if (groups)
		wxDELETEA(groups);

	numVerts = 0;
	numGroups = 0;
	numFaces = 0;
	vertIndex = 0;
	grpIndex = 0;

	if (!att)
		return;

	Model *m = NULL;
	if (att->model) {
		m = static_cast<Model*>(att->model);
		if (!m)
			return;

		AddCount(m);
	}

	// children:
	for (size_t i=0; i<att->children.size(); i++) {
		Model *mAtt = static_cast<Model*>(att->children[i]->model);
		if (mAtt)
			AddCount(mAtt);

		for (size_t j=0; i<att->children[i]->children.size(); i++) {
			Model *mAttChild = static_cast<Model*>(att->children[i]->children[j]->model);
			if (mAttChild)
				AddCount(mAttChild);
		}
	}

	numFaces = numVerts / 3;

	verts = new ModelData[numVerts];
	//indices = new float[numVerts];
	groups = new GroupData[numGroups];

	if (m)
		AddVertices(m);

	// children:
	for (size_t i=0; i<att->children.size(); i++) {
		Model *mAtt = static_cast<Model*>(att->children[i]->model);
		if (mAtt)
			AddVertices(mAtt);

		for (size_t j=0; i<att->children[i]->children.size(); i++) {
			Model *mAttChild = static_cast<Model*>(att->children[i]->children[j]->model);
			if (mAttChild)
				AddVertices(mAttChild);
		}
	}
}

void ExportM2to3DS(Model *m, const char *fn)
{
	/*
	ofstream f(fn, ios::out | ios::binary | ios::trunc);

	if (!f.is_open()) {
		wxLogMessage(_T("Error: Unable to open file '%s'. Could not export model."), fn);
		return;
	}

	unsigned short numVerts=0;
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				numVerts++;
			}
		}
	}

	MAX3DS_VERT *verts = new MAX3DS_VERT[numVerts];
	unsigned short numTris = numVerts / 3;
	MAX3DS_FACE *tris = new MAX3DS_FACE[numTris];

	int count=0;
	int tricount = 0;
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			int tri=0;
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];

				verts[count].x = m->vertices[a].x;
				verts[count].y = m->vertices[a].y;
				verts[count].z = m->vertices[a].z;
				count++;
				tri++;

				if (tri == 3) {
					tri = 0;
					tris[tricount].x = count-2;
					tris[tricount].y = count-1;
					tris[tricount].z = count;
					tricount++;
				}
			}
		}
	}

	// File structure
	MAX3DS_HEADER header;

	// Write header

	//reinterpret_cast<char *>(&numVerts), sizeof(numVerts));

	// ->
	MAX3DS_CHUNK mainObject;
	mainObject.id = EDIT3DS;

	// write header

	// -> ->
	MAX3DS_CHUNK subObjectEdit;
	subObjectEdit.id = EDIT_OBJECT;

	wxString modelName(m->name.c_str());
	modelName = modelName.AfterLast('\\');
	modelName.Append('\0');

	subObjectEdit.size += (unsigned int)modelName.length();

	// Write header

	// -> -> ->
	MAX3DS_CHUNK objMesh;
	objMesh.id = OBJ_MESH;

	// Write Header

	// -> -> -> ->
	MAX3DS_CHUNK objMeshVerts;
	objMeshVerts.id = MESH_VERTICES;
	//objMeshVerts.size += sizeof(numVerts)
	objMeshVerts.size += (numVerts * sizeof(MAX3DS_VERT));

	// Write Header
	// Write vertex data

	// -> -> -> ->
	MAX3DS_CHUNK objMeshTris;
	objMeshTris.id = MESH_FACEL1;
	//objMeshTris.size += sizeof(numTris);	// polygon indice size
	objMeshTris.size += (numTris * sizeof(MAX3DS_FACE)); //Polygon indice data size

	// Write Header
	// Write polygon data
	
	objMesh.size += objMeshVerts.size;
	objMesh.size += objMeshTris.size;
	subObjectEdit.size += objMesh.size;
	mainObject.size += subObjectEdit.size;
	header.primary.size += mainObject.size;


	// Write to our file
	f.write(reinterpret_cast<char *>(&header.primary.id), 2);
	f.write(reinterpret_cast<char *>(&header.primary.size), 4);
	f.write(reinterpret_cast<char *>(&header.version.id), 2);
	f.write(reinterpret_cast<char *>(&header.version.size), 4);
	f.write(reinterpret_cast<char *>(&header.nVersion), 4);

	f.write(reinterpret_cast<char *>(&mainObject.id), 2);
	f.write(reinterpret_cast<char *>(&mainObject.size), 4);
	f.write(reinterpret_cast<char *>(&subObjectEdit.id), 2);
	f.write(reinterpret_cast<char *>(&subObjectEdit.size), 4);

	char *model = (char *)modelName.c_str();
	f.write(model, (int)strlen(model));
	char nullByte = '\0';
	f.write(reinterpret_cast<char *>(&nullByte), 1);

	f.write(reinterpret_cast<char *>(&objMesh.id), 2);
	f.write(reinterpret_cast<char *>(&objMesh.size), 4);
	
	f.write(reinterpret_cast<char *>(&objMeshVerts.id), 2);
	f.write(reinterpret_cast<char *>(&objMeshVerts.size), 4);
	f.write(reinterpret_cast<char *>(verts), (numVerts * sizeof(MAX3DS_VERT)));
	f.write(reinterpret_cast<char *>(&objMeshTris.id), 2);
	f.write(reinterpret_cast<char *>(&objMeshTris.size), 4);
	f.write(reinterpret_cast<char *>(tris), (numTris * sizeof(MAX3DS_FACE)));

	f.close();

	wxDELETEA(verts);
	wxDELETEA(tris);

	/*
	f.write(reinterpret_cast<char *>(&header), sizeof(MAX3DS_HEADER));

	f.write(reinterpret_cast<char *>(&mainObject), sizeof(MAX3DS_CHUNK));
	f.write(reinterpret_cast<char *>(&subObjectEdit), sizeof(MAX3DS_CHUNK));
	f.write(reinterpret_cast<char *>(&objMesh), sizeof(MAX3DS_CHUNK));
	f.write(reinterpret_cast<char *>(&objMeshVerts), sizeof(MAX3DS_CHUNK));
	f.write(reinterpret_cast<char *>(verts), (numVerts * sizeof(MAX3DS_VERT)));
    f.write(reinterpret_cast<char *>(&objMeshTris), sizeof(MAX3DS_CHUNK));
	f.write(reinterpret_cast<char *>(tris), (numTris * sizeof(MAX3DS_FACE)));
	*/
	// */
}

void ExportM2toMS3D(Attachment *att, const char *fn)
{
	ofstream f(fn, ios::out | ios::binary | ios::trunc);

	if (!f.is_open()) {
		wxLogMessage(_T("Error: Unable to open file '%s'. Could not export model."), fn);
		return;
	}

	InitCommon(att);

	// Write the header
	ms3d_header_t header;
	strcpy(header.id, "MS3D000000");
	header.version = 4;

	// Header
	f.write(reinterpret_cast<char *>(&header), sizeof(ms3d_header_t));
	// Vertex Count
	f.write(reinterpret_cast<char *>(&numVerts), sizeof(numVerts));
	
	// Write Vertex data?
	for (unsigned int i=0; i<numVerts; i++) {
		ms3d_vertex_t vert;
		vert.boneId = -1;
		vert.flags = SELECTED;
		vert.referenceCount = 0; // what the?
		vert.vertex[0] = verts[i].vertex.x;
		vert.vertex[1] = verts[i].vertex.y;
		vert.vertex[2] = verts[i].vertex.z;
		f.write(reinterpret_cast<char *>(&vert), sizeof(ms3d_vertex_t));
	}
	// ---------------------------

	// Triangle Count
	f.write(reinterpret_cast<char *>(&numFaces), 2);

	// Write Triangle Data?
	for (unsigned int i=0; i<numVerts; i+=3) {
		ms3d_triangle_t tri;
		tri.flags = SELECTED;
		tri.groupIndex = (unsigned char)verts[i].groupIndex;
		tri.smoothingGroup = 1;

		for (int j=0; j<3; j++) {
			tri.vertexIndices[j] = i+j;
			tri.s[j] = verts[i+j].tu;
			tri.t[j] = verts[i+j].tv;
			
			tri.vertexNormals[j][0] = verts[i+j].normal.x;
			tri.vertexNormals[j][1] = verts[i+j].normal.y;
			tri.vertexNormals[j][2] = verts[i+j].normal.z;
		}

		f.write(reinterpret_cast<char *>(&tri), sizeof(ms3d_triangle_t));
	}
	// ---------------------------

	// Number of groups
	f.write(reinterpret_cast<char *>(&numGroups), 2);

	unsigned short indiceCount = 0;
	for (unsigned short i=0; i<numGroups; i++) {
		wxString groupName(wxString::Format("Geoset_%i", i));

		const char flags = SELECTED;
		f.write(&flags, 1);

		char name[32];
		strcpy(name, groupName.c_str());
		f.write(name, 32);

		unsigned short faceCount = groups[i].p.indexCount / 3;
		f.write(reinterpret_cast<char *>(&faceCount), 2);
		
		for (int k=0; k<faceCount; k++) {
			//triIndices[k] = indiceCount;
			f.write(reinterpret_cast<char *>(&indiceCount), 2);
			indiceCount++;
		}

		f.write(reinterpret_cast<char *>(&i), 1);
	}

	// Number of materials (pretty much identical to groups, each group has its own material)
	f.write(reinterpret_cast<char *>(&numGroups), 2);
	
	for (unsigned short i=0; i<numGroups; i++) {
		wxString matName(wxString::Format("Material_%i", i));

		ModelRenderPass p = groups[i].p;
		if (p.init(groups[i].m)) {
			ms3d_material_t mat;
			memset(mat.alphamap, '\0', 128);
			//mat.alphamap = (char)0;
			mat.ambient[0] = 0.7f;
			mat.ambient[1] = 0.7f;
			mat.ambient[2] = 0.7f;
			mat.ambient[3] = 1.0f;
			mat.diffuse[0] = p.ocol.x;
			mat.diffuse[1] = p.ocol.y;
			mat.diffuse[2] = p.ocol.z;
			mat.diffuse[3] = p.ocol.w;
			mat.emissive[0] = p.ecol.x;
			mat.emissive[1] = p.ecol.y;
			mat.emissive[2] = p.ecol.z;
			mat.emissive[3] = p.ecol.w;

			strcpy(mat.name, matName.c_str());

			if (p.useEnvMap) {
				mat.shininess = 30.0f;
				mat.mode = 1;
			} else {
				mat.shininess = 0.0f;
				mat.mode = 0;
			}
			
			mat.specular[0] = 0.0f;
			mat.specular[1] = 0.0f;
			mat.specular[2] = 0.0f;
			mat.specular[3] = 1.0f;
			wxString texName(fn);
			texName = texName.AfterLast('\\').BeforeLast('.');
			texName << "_" << p.tex << ".png";
			strcpy(mat.texture, texName.c_str());
			mat.transparency = p.ocol.w;

			f.write(reinterpret_cast<char *>(&mat), sizeof(ms3d_material_t));

			wxString texFilename(fn);
			texFilename = texFilename.BeforeLast('\\');
			texFilename += '\\';
			texFilename += texName;
			SaveTexture(texFilename);
		}
	}

	// save some keyframer data
	float fps = 0.0f; //m->anims[m->anim].playSpeed;
	float fCurTime = 0.0f;
	int totalFrames = 0;//m->anims[m->anim].timeEnd - m->anims[m->anim].timeStart;

	f.write(reinterpret_cast<char *>(&fps), 4);
	f.write(reinterpret_cast<char *>(&fCurTime), 4);
	f.write(reinterpret_cast<char *>(&totalFrames), 4);
	
	
	// number of joints
	unsigned short numJoints = 0;//(unsigned short)m->header.nBones;

	f.write(reinterpret_cast<char *>(&numJoints), 2);

	/*
	for (int i=0; i<numJoints; i++) {
		ms3d_joint_t joint;

		joint.flags = SELECTED;
		memset(joint.name, '\0', 32);
		memset(joint.parentName, '\0', 32);
		
		joint.rotation[0] = m->bones[i].pivot.x;
		joint.rotation[1] = m->bones[i].pivot.y;
		joint.rotation[2] = m->bones[i].pivot.z;

		joint.position[0] = m->bones[i].transPivot.x;
		joint.position[1] = m->bones[i].transPivot.y;
		joint.position[2] = m->bones[i].transPivot.z;

		joint.numKeyFramesRot = (unsigned short)m->bones[i].boneDef.rotation.nKeys;
		joint.numKeyFramesTrans = (unsigned short)m->bones[i].boneDef.translation.nKeys;

		f.write(reinterpret_cast<char *>(&joint), sizeof(ms3d_joint_t));

		if (joint.numKeyFramesRot > 0) {
			ms3d_keyframe_rot_t *keyFramesRot = new ms3d_keyframe_rot_t[joint.numKeyFramesRot];
			for (int j=0; j<joint.numKeyFramesRot; j++) {
				keyFramesRot[j].time = 0;
				keyFramesRot[j].rotation[0] = m->bones[i].rot.getValue(0,0).x;
				keyFramesRot[j].rotation[1] = m->bones[i].rot.getValue(0,0).y;
				keyFramesRot[j].rotation[2] = m->bones[i].rot.getValue(0,0).z;
			}

			f.write(reinterpret_cast<char *>(keyFramesRot), sizeof(ms3d_keyframe_rot_t) * joint.numKeyFramesRot);
			wxDELETEA(keyFramesRot);
		}

		if (joint.numKeyFramesTrans > 0) {
			ms3d_keyframe_pos_t *keyFramesTrans = new ms3d_keyframe_pos_t[joint.numKeyFramesTrans];
			for (int j=0; j<joint.numKeyFramesTrans; j++) {
				keyFramesTrans[j].time = 0;
				keyFramesTrans[j].position[0] = m->bones[i].trans.getValue(0,0).x;
				keyFramesTrans[j].position[1] = m->bones[i].trans.getValue(0,0).y;
				keyFramesTrans[j].position[2] = m->bones[i].trans.getValue(0,0).z;
			}

			f.write(reinterpret_cast<char *>(keyFramesTrans), sizeof(ms3d_keyframe_pos_t) * joint.numKeyFramesTrans);
			wxDELETEA(keyFramesTrans);
		}
	}
	*/

	f.close();

	if (verts)
		wxDELETEA(verts);
	if (groups)
		wxDELETEA(groups);
}

void ExportM2toLWO(Model *m, const char *fn)
{
	int i32;
	uint32 u32;
	float f32;
	uint16 u16;
	unsigned char ub;

	ofstream f(fn, ios::out | ios::binary | ios::trunc);

	if (!f.is_open()) {
		wxLogMessage(_T("Error: Unable to open file '%s'. Could not export model."), fn);
		return;
	}

	unsigned int fileLen = 0;

	f.write("FORM", 4);
	f.write(reinterpret_cast<char *>(&fileLen), 4);

	f.write("LWOB", 4);
	fileLen += 4;


	// --
	// POINTS CHUNK, this is the vertice data
	uint32 pointsSize = 0;
	f.write("PNTS", 4);
	u32 = reverse_endian<uint32>(pointsSize);
	f.write(reinterpret_cast<char *>(&u32), 4);
	fileLen += 8;

	// output all the vertice data
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];
				Vec3D vert;
				vert.x = reverse_endian<float>(m->vertices[a].x);
				vert.y = reverse_endian<float>(m->vertices[a].y);
				vert.z = reverse_endian<float>(m->vertices[a].z);
				f.write(reinterpret_cast<char *>(&vert.x), 4);
				f.write(reinterpret_cast<char *>(&vert.y), 4);
				f.write(reinterpret_cast<char *>(&vert.z), 4);
				fileLen += 12;
				pointsSize += 12;

				numVerts++;
			}
			numGroups++;
		}
	}
	// ================


	// --
	// SURFACE CHUNK,
	uint32 surfaceSize = 0;
	f.write("SRFS", 4);
	f.write(reinterpret_cast<char *>(&surfaceSize), 4);
	fileLen += 8;

	wxString surfName;
	int surfaceCounter = 0;

	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			surfName = "geoset_";
			surfName << surfaceCounter;
			surfaceCounter++;
			
			surfName.Append('\0');
			if (fmod((float)surfName.length(), 2.0f) > 0)
				surfName.Append('\0');

			f.write(surfName.c_str(), (int)surfName.length());
			
			fileLen += (uint32)surfName.length();
			surfaceSize += (uint32)surfName.length();
		}
	}
	
	f.seekp(-4 - surfaceSize, ios::cur);
	u32 = reverse_endian<uint32>(surfaceSize);
	f.write(reinterpret_cast<char *>(&u32), 4);
	f.seekp(0, ios::end);
	// =================

	// --
	// POLYGON CHUNK
	int32 polySize = (numVerts / 3) * sizeof(POLYCHUNK);

	f.write("POLS", 4);
	i32 = reverse_endian<int32>(polySize);
	f.write(reinterpret_cast<char *>(&i32), 4);
	fileLen += 8; // an extra 4 bytes for chunk size

	uint16 counter=0;
	int16 surfCounter=0;
	POLYCHUNK tri;
	
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {

			surfCounter++;

			for (unsigned int k=0; k<p.indexCount; k+=3) {
				u16 = 3;
				tri.numVerts = ByteSwap16(u16);
				for (uint16 b=0; b<3; b++) {
					tri.indice[b] = ByteSwap16(counter);
					counter++;
				}
				tri.surfIndex = ByteSwap16(surfCounter);

				f.write(reinterpret_cast<char *>(&tri), sizeof(POLYCHUNK));

				fileLen += 10;
			}
		}
	}
	
	// Now lets go back and correct our data lengths
	//f.seekp(-4 - polySize, ios::cur);
	//i32 = reverse_endian<int32>(polySize);
	//f.write(reinterpret_cast<char *>(&i32), 4);
	// ========


	// Now we need to write our surface definitions (ie. RenderPass data)
	// --------------------------
	//f.seekp(0, ios::end);

	surfaceCounter = 0;

	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			uint32 surfaceDefSize = 0;
			f.write("SURF", 4);
			f.write(reinterpret_cast<char *>(&surfaceDefSize), 4);
			fileLen += 8;

			// Surface name
			surfName = "geoset_";
			surfName << surfaceCounter;
			surfaceCounter++;
			
			surfName.Append('\0');
			if (fmod((float)surfName.length(), 2.0f) > 0)
				surfName.Append('\0');

			f.write(surfName.data(), (int)surfName.length());
			
			fileLen += (uint32)surfName.length();
			surfaceDefSize += (uint32)surfName.length();

			// Surface Attributes
			// COLOUR, size 4, bytes 2
			f.write("COLR", 4);
			u16 = 4;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);
			
			// value
			ub = (unsigned char)(p.ocol.x * 255);
			f.write(reinterpret_cast<char *>(&ub), 1);
			ub = (unsigned char)(p.ocol.y * 255);
			f.write(reinterpret_cast<char *>(&ub), 1);
			ub = (unsigned char)(p.ocol.z * 255);
			f.write(reinterpret_cast<char *>(&ub), 1);
			ub = '\0';
			f.write(reinterpret_cast<char *>(&ub), 1);

			fileLen += 10;
			surfaceDefSize += 10;
			
			// FLAGS
			f.write("FLAG", 4);
			// size
			u16 = 2;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			// value
			u16 = 0;
			if (!p.cull)
				u16 &= SUF_DOUBLESIDED;
			if (p.blendmode>2)
				u16 &= SUF_ADDITIVE;
			if (p.blendmode==1)
				u16 &= SUF_EDGETRANSPARENT;
			
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			fileLen += 8;
			surfaceDefSize += 8;

			// GLOSSINESS
			f.write("GLOS", 4);
			// size
			u16 = 2;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			// Value
			// try 50% gloss for 'relfection surfaces
			if (p.useEnvMap)
				u16 = 128;
			else
				u16 = 0;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			fileLen += 8;
			surfaceDefSize += 8;
			
			if (p.useEnvMap) {
				// REFLECTION
				f.write("FRFL", 4);
				// size
				u16 = 4;
				u16 = ByteSwap16(u16);
				f.write(reinterpret_cast<char *>(&u16), 2);

				// value
				f32 = 0.2f;
				f32 = reverse_endian<float>(f32);
				f.write(reinterpret_cast<char *>(&f32), 4);

				fileLen += 10;
				surfaceDefSize += 10;
				
				// REFLECTION
				f.write("RFLT", 4);
				// size
				u16 = 2;
				u16 = ByteSwap16(u16);
				f.write(reinterpret_cast<char *>(&u16), 2);

				// value
				u16 = 1;
				u16 = ByteSwap16(u16);
				f.write(reinterpret_cast<char *>(&u16), 2);

				fileLen += 8;
				surfaceDefSize += 8;
			}
			

			// TRANSPARENCY
			f.write("FTRN", 4);
			u16 = 4; // size
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			// value
			f32 = p.ocol.w;
			f32 = reverse_endian<float>(f32);
			f.write(reinterpret_cast<char *>(&f32), 4);

			fileLen += 10;
			surfaceDefSize += 10;

			// TEXTURE FLAGS
			f.write("TFLG", 4);
			// size
			u16 = 2;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			// value
			u16 = 0; // don't know the flag info yet
			if (p.Trans)
				u16 &= TXF_PIXBLEND;
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);

			fileLen += 8;
			surfaceDefSize += 8;

			wxString texName(fn);
			texName = texName.AfterLast('\\').BeforeLast('.');
			texName << "_" << p.tex << ".tga" << '\0';
			if (fmod((float)texName.length(), 2.0f) > 0)
				texName.Append('\0');

			// TEXTURE
			f.write("TIMG", 4);
			u16 = (unsigned short)texName.length();
			u16 = ByteSwap16(u16);
			f.write(reinterpret_cast<char *>(&u16), 2);
			f.write(texName.data(), (int)texName.length());

			fileLen += (unsigned int)texName.length() + 6;
			surfaceDefSize += (unsigned int)texName.length() + 6;

			// update the chunks length
			f.seekp(-4 - surfaceDefSize, ios::cur);
			u32 = reverse_endian<uint32>(surfaceDefSize);
			f.write(reinterpret_cast<char *>(&u32), 4);
			f.seekp(0, ios::end);

			wxString texFilename(fn);
			texFilename = texFilename.BeforeLast('\\');
			texFilename += '\\';
			texFilename += texName;
			SaveTexture(texFilename);
		}
	}
	
	f.seekp(4,ios::beg);
	u32 = reverse_endian<uint32>(fileLen);
	f.write(reinterpret_cast<char *>(&u32), 4);
	
	f.seekp(16, ios::beg);
	u32 = reverse_endian<uint32>(pointsSize);
	f.write(reinterpret_cast<char *>(&u32), 4);
	// ===========================


	f.close();
}

void ExportM2toOBJ(Model *m, const char *fn)
{
	// Open file
	ofstream f(fn, ios_base::out | ios_base::trunc);

	if (!f.is_open()) {
		wxLogMessage(_T("Error: Unable to open file '%s'. Could not export model."), fn);
		return;
	}

	wxString matName(fn);
	matName = matName.AfterLast('\\').BeforeLast('.');
	matName << ".mtl";

	ofstream fm(matName.c_str(), ios_base::out | ios_base::trunc);
	
	// output all the vertice data
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			f << "# Chunk Indice Count: " << p.indexCount << endl;

			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];
				f << "v " << m->vertices[a].x << " " << m->vertices[a].y << " " << m->vertices[a].z << " " << "1.0" << endl;
			}

			wxString texName(fn);
			texName = texName.AfterLast('\\').BeforeLast('.');
			texName << "_" << p.tex << ".tga";
			fm << "newmtl " << "Material_" << p.geoset << endl;
			fm << "Ka " << 0.7f << " " << 0.7f << " " << 0.7f << endl;
			fm << "Kd " << p.ocol.x << " " << p.ocol.y << " " << p.ocol.z << endl;
			fm << "Ks " << p.ecol.x << " " << p.ecol.y << " " << p.ecol.z << endl;
			fm << "Ns " << p.ocol.w << endl;
			fm << "map_Kd " << texName.c_str() << endl << endl;

			wxString texFilename(fn);
			texFilename = texFilename.BeforeLast('\\');
			texFilename += '\\';
			texFilename += texName;
			SaveTexture(texFilename);
		}
	}

	fm.close();

	f << "mtllib " << matName.c_str() << endl;

	// output all the texture coordinate data
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];
		// we don't want to render completely transparent parts
		if (p.init(m)) {
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];
				f << "vt " << m->origVertices[a].texcoords.x << " " << m->origVertices[a].texcoords.y << endl;
			}
		}
	}

	// output all the vertice normals data
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];
		if (p.init(m)) {
			for (size_t k=0, b=p.indexStart; k<p.indexCount; k++,b++) {
				uint16 a = m->indices[b];
				f << "vn " << m->origVertices[a].normal.x << " " << m->origVertices[a].normal.y << " " << m->origVertices[a].normal.z << endl;
			}
		}
	}

	int counter=1;
	// output the indice data
	for (size_t i=0; i<m->passes.size(); i++) {
		ModelRenderPass &p = m->passes[i];

		if (p.init(m)) {
			f << "g Geoset_" << p.geoset << endl;
			f << "s 1" << endl;
			f << "usemtl Material_" << p.geoset << endl;
			for (unsigned int k=0; k<p.indexCount; k+=3) {
				f << "f " << counter << " " << counter+1 << " " << counter+2 << endl;
				counter+=3;
			}
		}
	}

	f << "# Total Indices: " << counter << endl;
	
	// Close file
	f.close();
}

void ExportM2toRAW(Model *m)
{
	
}