
#ifndef MODELVIEWER_H
#define MODELVIEWER_H

// wx
#include "wx/wx.h"
#include "wx/filename.h"
#include "wx/fileconf.h"
#include "wx/treectrl.h"
#include "wx/colordlg.h"
#include "wx/msgdlg.h"

//wxAUI
#include "wxaui_manager.h"

// Our files
#include "modelcanvas.h"
#include "animcontrol.h"
#include "charcontrol.h"
#include "viewcontrol.h"
#include "lightcontrol.h"
#include "modelcontrol.h"
#include "aboutframe.h"
#include "imagecontrol.h"
#include "mpq.h"
#include "util.h"
#include "gifexporter.h"
#include "effects.h"
#include "arrows.h"
#include "modelexport.h"

#include "enums.h"

#include "shader.h"

class ModelViewer: public wxFrame
{    
    DECLARE_CLASS(ModelViewer)
    DECLARE_EVENT_TABLE()

	std::set<FileTreeItem> filelist;
	std::vector<MPQArchive*> archives;

public:

	// Constructor + Deconstructor
	ModelViewer(wxWindow* parent, wxWindowID id = -1, const wxString& caption = _T("ModelViewer"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU);
	~ModelViewer();

	// our class objects
	AnimControl *animControl;
	ModelCanvas *canvas;
	CharControl *charControl;
	ViewControl *viewControl;
	GifExporter *gifExporter;
	EnchantsDialog *enchants;
	LightControl *lightControl;
	ModelControl *modelControl;
	ArrowControl *arrowControl;
	ImageControl *imageControl;

	wxTreeCtrl *fileTree;

	//wxWidget objects
	wxMenuBar *menuBar;
	wxMenu *charMenu, *viewMenu, *optMenu, *lightMenu;
	
	// wxAUI - new docking lib
	wxFrameManager interfaceManager;

	// Boolean flags
	bool isModel;
	bool isChar;
	bool isWMO;

	// Initialising related functions
	void InitMenu();
	void InitObjects();
	void Init();
	void InitDocking();
	void InitDatabase();
	void UpdateCanvasPtrs();

	// Save and load various settings between sessions
	void LoadSession();
	void SaveSession();
	// Save and load the GUI layout
	void LoadLayout();
	void SaveLayout();
	void ResetLayout();
	// save + load character *.CHR files
	void LoadChar(const char *fn);
	void SaveChar(const char *fn);

	void LoadModel(const wxString fn);
	void LoadNPC(unsigned int modelid);

	// Window GUI event related functions
	//void OnIdle();
	void OnClose(wxCloseEvent &event);
	void OnSize(wxSizeEvent &event);
    void OnExit(wxCommandEvent &event);
	void OnTreeSelect(wxTreeEvent &event);

    // menu commands
	void OnToggleDock(wxCommandEvent &event);
	void OnToggleCommand(wxCommandEvent &event);
	void OnSetColor(wxCommandEvent &event);
	void OnEffects(wxCommandEvent &event);
	void OnLightMenu(wxCommandEvent &event);
	void OnCamMenu(wxCommandEvent &event);

	// Wrapper function for character stuff (forwards events to charcontrol)
	void OnSetEquipment(wxCommandEvent &event);
	void OnCharToggle(wxCommandEvent &event);

	void OnMount(wxCommandEvent &event);
	void OnSave(wxCommandEvent &event);
	void OnBackground(wxCommandEvent &event);
	void OnLanguage(wxCommandEvent &event);
	void OnAbout(wxCommandEvent &event);
	void OnCanvasSize(wxCommandEvent &event);
	void OnTest(wxCommandEvent &event);
	void OnExport(wxCommandEvent &event);

	void UpdateControls();
   
	Vec3D DoSetColor(const Vec3D &defColor);
};


bool filterModels(string);
bool filterNpcs(string);

#endif

